/**
 * vue-promised v1.1.0
 * (c) 2019 Eduardo San Martin Morote <posva13@gmail.com>
 * @license MIT
 */

'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function assert (condition, message) {
  if ("development" !== 'production' && !condition) {
    // useful when testing
    // console.warn(`[vue-promised] ${message}`)
    throw new Error(("[vue-promised] " + message))
  }
}

var Promised = {
  props: {
    tag: {
      type: String,
      default: 'span',
    },
    promise: {
      // allow polyfied Promise
      validator: function (p) { return p && typeof p.then === 'function' && typeof p.catch === 'function'; },
    },
    pendingDelay: {
      type: [Number, String],
      default: 200,
    },
  },

  data: function () { return ({
    resolved: false,
    data: null,
    error: null,

    isDelayElapsed: false,
  }); },

  render: function render (h) {
    if (this.$scopedSlots.combined) {
      var node = this.$scopedSlots.combined({
        isPending: !this.resolved,
        isDelayOver: this.isDelayElapsed,
        data: this.data,
        error: this.error,
      });
      assert(
        (Array.isArray(node) && node.length) || node,
        'Provided "combined" scoped slot cannot be empty'
      );
      return Array.isArray(node) ? convertVNodeArray(h, this.tag, node) : node
    }

    if (this.error) {
      assert(
        this.$scopedSlots.rejected,
        'No slot "rejected" provided. Cannot display the error'
      );
      var node$1 = this.$scopedSlots.rejected(this.error);
      assert(
        (Array.isArray(node$1) && node$1.length) || node$1,
        'Provided slot "rejected" is empty. Cannot display the error'
      );
      return Array.isArray(node$1) ? convertVNodeArray(h, this.tag, node$1) : node$1
    }

    if (this.resolved) {
      if (this.$scopedSlots.default) {
        var node$2 = this.$scopedSlots.default(this.data);
        assert(
          (Array.isArray(node$2) && node$2.length) || node$2,
          'Provided default scoped-slot is empty. Cannot display the data'
        );
        return Array.isArray(node$2) ? convertVNodeArray(h, this.tag, node$2) : node$2
      }
      // NOTE: for Vue < 2.6.0
      var defaultSlot = this.$slots.default;
      assert(defaultSlot, 'No default slot provided. Cannot display the data');
      /* istanbul ignore next */
      assert(
        defaultSlot.length,
        'Provided default slot is empty. Cannot display the data'
      );
      /* istanbul ignore next */
      return convertVNodeArray(h, this.tag, defaultSlot)
    }

    if (!this.isDelayElapsed) { return h() }

    var pendingSlot = this.$slots.pending;
    assert(
      pendingSlot,
      'No "pending" slot provided. Cannot display pending state'
    );
    assert(
      pendingSlot.length,
      'Provided "pending" slot is empty. Cannot display pending state'
    );
    return convertVNodeArray(h, this.tag, pendingSlot)
  },

  watch: {
    promise: {
      handler: function handler (promise) {
        var this$1 = this;

        if (!promise) { return }
        this.resolved = false;
        this.error = null;
        this.setupDelay();
        promise
          .then(function (data) {
            // ensure we are dealing with the same promise
            if (this$1.promise === promise) {
              this$1.data = data;
              this$1.resolved = true;
            }
          })
          .catch(function (err) {
            // ensure we are dealing with the same promise
            if (this$1.promise === promise) {
              this$1.error = err;
              this$1.resolved = true;
            }
          });
      },
      immediate: true,
    },
  },

  methods: {
    setupDelay: function setupDelay () {
      var this$1 = this;

      if (this.pendingDelay > 0) {
        this.isDelayElapsed = false;
        if (this.timerId) { clearTimeout(this.timerId); }
        this.timerId = setTimeout(
          function () { return (this$1.isDelayElapsed = true); },
          this.pendingDelay
        );
      } else {
        this.isDelayElapsed = true;
      }
    },
  },
};

function convertVNodeArray (h, wrapperTag, nodes) {
  // for arrays and single text nodes
  if (nodes.length > 1 || !nodes[0].tag) { return h(wrapperTag, {}, nodes) }
  return nodes[0]
}

exports.Promised = Promised;
